package filepack

import (
	"log"
	"testing"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/loggermdl"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/filemdl"
)

func TestCreatePack(t *testing.T) {
	// get file list
	pathList, err := filemdl.ListFileRecursively("/home/vivekn/fdb_data/Candidates")
	if err != nil {
		log.Fatal(err)
	}

	// create pack
	err = CreatePackFile(pathList, "/home/vivekn/fdb_data/Candidates.pack")
	if err != nil {
		log.Fatal(err)
	}
}

func TestInsertFiles(t *testing.T) {
	// get file list
	fileList := []string{"/home/vivekn/cmd"}
	err := InsertFiles("/home/vivekn/fdb_data/Candidates.pack", fileList)

	if err != nil {
		log.Fatal(err)
	}
}
func TestGetFileData(t *testing.T) {
	// data, err := GetFileDataFromPack("/home/vivekn/fdb_data/Candidates.pack", "/home/vivekn/fdb_data/Candidates/10008")
	data, err := GetFileDataFromPack("/home/vivekn/fdb_data/Candidates.pack", "/home/vivekn/cmd")

	if err != nil {
		log.Fatal(err)
	}
	loggermdl.LogDebug("data", string(data))
}
func TestDeleteFile(t *testing.T) {
	// create pack
	err := DeleteFileFromPack("/home/vivekn/fdb_data/Candidates.pack", "/home/vivekn/fdb_data/Candidates/10008")
	if err != nil {
		log.Fatal(err)
	}
}

func BenchmarkCreatePack(b *testing.B) {
	// get file list
	pathList, err := filemdl.ListFileRecursively("/home/vivekn/fdb_data/Candidates")
	if err != nil {
		log.Fatal(err)
	}
	for index := 0; index < b.N; index++ {
		// create pack
		err = CreatePackFile(pathList, "/home/vivekn/fdb_data/Candidates.pack")
		if err != nil {
			log.Fatal(err)
		}
	}

}

func BenchmarkGetFileDataListFromPack(b *testing.B) {
	pathList := []string{"/home/vivekn/fdb_data/Candidates/file200", "/home/vivekn/fdb_data/Candidates/file100", "/home/vivekn/fdb_data/Candidates/file202"}
	for index := 0; index < b.N; index++ {
		_, err := GetFileDataListFromPack("/home/vivekn/fdb_data/Candidates.pack", pathList)
		if err != nil {
			log.Fatal(err)
		}
	}

}

func BenchmarkGetFileDataFromPack(b *testing.B) {
	// pathList := []string{"/home/vivekn/fdb_data/Candidates/file200", "/home/vivekn/fdb_data/Candidates/file100", "/home/vivekn/fdb_data/Candidates/file202"}
	for index := 0; index < b.N; index++ {
		_, err := GetFileDataFromPack("/home/vivekn/fdb_data/Candidates.pack", "/home/vivekn/fdb_data/Candidates/file1")
		if err != nil {
			log.Fatal(err)
		}
	}

}

func TestFastWriteFile(t *testing.T) {
	err := filemdl.FastWriteFile("C:/Users/shridharp/Desktop/Test/testF.txt", []byte("how are you"), true, true)
	if err != nil {
		log.Fatal(err)
	}
}
