package routebuildermdl

import (
	"strconv"
	"strings"
	"time"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/cachemdl"
	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/errormdl"
	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/loggermdl"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/servicebuildermdl"
	"github.com/tidwall/gjson"
	"github.com/zhouzhuojie/conditions"
)

var formConditions cachemdl.FastCacheHelper

func init() {
	formConditions.Setup(1000, time.Hour*24, time.Hour*24)
}

// func dynamicFormEvalution(ab *servicebuildermdl.AbstractBusinessLogicHolder, rs gjson.Result) (interface{}, error) {
// 	if ab == nil {
// 		return nil, nil
// 	}
// 	eval := ab.GetDynamicEvalData()
// 	if eval == nil {
// 		return nil, nil
// 	}
// 	loggermdl.LogDebug("EVAL1:", eval)
// 	configs := rs.Get("conditions").Array()
// 	loggermdl.LogDebug("Conditions:", configs)
// 	if rs.String() == "" || len(configs) == 0 {
// 		return nil, nil
// 	}
// 	loggermdl.LogDebug("CONFIG:", configs)
// 	currentPage := rs.Get("currentPage").String()
// 	if currentPage == "" {
// 		loggermdl.LogError("No current page found")
// 		return nil, errormdl.Wrap("No current page found")
// 	}
// 	loggermdl.LogDebug("CURRRENT_PAGE:", currentPage)

// 	currentPageIndex := -1
// 	isCurrentPageIsNC := false
// 	isEndPage := false
// 	endPageObj := gjson.Result{}
// 	for i, obj := range configs {
// 		loggermdl.LogDebug("INDEX:" + strconv.Itoa(i))
// 		condition := obj.Get("condition").String()
// 		loggermdl.LogDebug("condition:", condition)
// 		pageID := obj.Get("pageID").String()
// 		loggermdl.LogDebug("PAGE_ID:", pageID)
// 		noCondition := obj.Get("noCondition").Bool()
// 		loggermdl.LogDebug("NO_CONDITION:", noCondition)
// 		if pageID == currentPage {
// 			loggermdl.LogDebug("MATCH PAGE id", i)
// 			isEndPage = obj.Get("isEndPage").Bool()
// 			if isEndPage {
// 				endPageObj = obj
// 			}
// 			if noCondition {
// 				isCurrentPageIsNC = true
// 			}
// 			currentPageIndex = i
// 			continue
// 		}
// 		var expr conditions.Expr
// 		var err error
// 		if val, ok := formConditions.Get(condition); ok {
// 			loggermdl.LogDebug("FOUND in CACHE:", val)
// 			expr, ok = val.(conditions.Expr)
// 			if !ok {
// 				loggermdl.LogDebug("PARSING CONDITION")
// 				parse := conditions.NewParser(strings.NewReader(condition))
// 				expr, err = parse.Parse()
// 				if errormdl.CheckErr(err) != nil {
// 					ab.SetErrorCode(errormdl.CONDITIONPARSEERROR)
// 					loggermdl.LogError(err)
// 					return nil, errormdl.CheckErr(err)
// 				}
// 				formConditions.SetNoExpiration(condition, expr)
// 			}
// 		} else {
// 			loggermdl.LogDebug("NOT FOUND IN CACHE:", i)

// 			parse := conditions.NewParser(strings.NewReader(condition))
// 			expr, err = parse.Parse()
// 			if errormdl.CheckErr(err) != nil {
// 				ab.SetErrorCode(errormdl.CONDITIONEVALERROR)
// 				loggermdl.LogError(err)
// 				return nil, errormdl.CheckErr(err)
// 			}

// 			formConditions.SetNoExpiration(condition, expr)
// 		}
// 		result, evalErr := conditions.Evaluate(expr, eval)
// 		if errormdl.CheckErr(evalErr) != nil {
// 			continue
// 		}
// 		if result {
// 			return obj.Value(), nil
// 		}
// 	}

// 	if isEndPage {
// 		return endPageObj.Value(), nil
// 	}

// 	for i, obj := range configs {
// 		if isCurrentPageIsNC && i <= currentPageIndex {
// 			continue
// 		}
// 		noCondition := obj.Get("noCondition").Bool()
// 		if noCondition {
// 			return obj.Value(), nil
// 		}
// 	}
// 	ab.SetErrorCode(errormdl.NEXTFORMEVALERROR)
// 	return nil, errormdl.Wrap("NO CONDITION MATCHED")
// }

// dynamic form evaluation
// End page and sequential wizard handling is on client side

// look into conditions array and return a page matching the condition
// if no condition matched, return first no condition page from current page position
// if no conditions matched and we do not have any no condition page, return No_Conditions_Matched error

func dynamicFormEvalution(ab *servicebuildermdl.AbstractBusinessLogicHolder, rs gjson.Result) (interface{}, error) {
	if ab == nil {
		return nil, nil
	}
	// get data to be evaluated ffrom request body
	inputData := ab.GetDynamicEvalData()
	if inputData == nil {
		return nil, nil
	}

	// get conditions
	conditionsArray := rs.Get("conditions").Array()
	lenConditions := len(conditionsArray)
	if rs.String() == "" || lenConditions == 0 {
		loggermdl.LogInfo("No conditins found!")
		return nil, nil
	}

	// get current page
	currentPage := rs.Get("currentPage").String()
	if currentPage == "" {
		loggermdl.LogError("No current page found")
		return nil, errormdl.Wrap("No current page found")
	}
	loggermdl.LogDebug("currentPage:", currentPage)

	currentPageIndex := -1
	noConditionPage := gjson.Result{}
	isNCSet := false

	for i, obj := range conditionsArray {

		pageID := obj.Get("pageID").String()
		noCondition := obj.Get("noCondition").Bool()
		if pageID == currentPage {
			// return nothing if current page is end page, end page handling is on clinet side
			if obj.Get("isEndPage").Bool() {
				return nil, nil
			}
			currentPageIndex = i
			continue
		}

		condition := obj.Get("condition").String()
		loggermdl.LogDebug("condition:", condition)

		// store no condition page so that it can be used if no condition is matched
		if currentPageIndex != -1 && i > currentPageIndex && noCondition && !isNCSet {
			loggermdl.LogInfo("No condition page at ", i)
			noConditionPage, isNCSet = obj, true
			// continue
		}

		if noCondition {
			// important, if condition is empty, conditions package throws parsing error
			continue
		}

		var expr conditions.Expr
		var err error

		// use cache to store parsed expression
		if ex, found := formConditions.Get(condition); !found {
			loggermdl.LogInfo("Not found in cache")
			expr, err = conditions.NewParser(strings.NewReader(condition)).Parse()
			if errormdl.CheckErr(err) != nil {
				ab.SetErrorCode(errormdl.CONDITIONPARSEERROR)
				loggermdl.LogError(err)
				return nil, errormdl.Wrap("error parsing condition at " + strconv.Itoa(i) + " - " + err.Error())
			}

			formConditions.SetNoExpiration(condition, expr)
			loggermdl.LogInfo("Added to cache")
		} else {
			loggermdl.LogInfo("Added to cache")
			exCasted, casted := ex.(conditions.Expr)
			if !casted {
				loggermdl.LogError("failed to cast expression for condition ", condition)
				continue
			}

			expr = exCasted
		}

		result, evalErr := conditions.Evaluate(expr, inputData)
		if errormdl.CheckErr(evalErr) != nil {
			// contunue to next condition
			return nil, errormdl.Wrap("error evaluating condition at " + strconv.Itoa(i) + " - " + evalErr.Error())
		}
		if result {
			// evaluated successffully
			return obj.Value(), nil
		}
	}

	// no condition matched, return first no condition page after current page
	loggermdl.LogInfo("No condition matched")
	if isNCSet {
		return noConditionPage.Value(), nil
	}
	{
	}
	ab.SetErrorCode(errormdl.NEXTFORMEVALERROR)
	return nil, errormdl.Wrap("NO CONDITION MATCHED")
}
