//  +build !fasthttp

package routebuildermdl

import (
	"io/ioutil"
	"net/http"
	"strings"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/authmdl/roleenforcemdl"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/statemdl"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/authmdl/jwtmdl"
	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/errormdl"
	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/loggermdl"
	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/servicebuildermdl"
	"github.com/gin-gonic/gin"
	version "github.com/hashicorp/go-version"
	"github.com/tidwall/gjson"
)

// Init routing init
func Init(o, r, c *gin.RouterGroup, JWTKey string) {
	o.POST("/mql/state", statemdl.StateHandler)
	o.POST("/mql", OpenHandler)
	r.POST("/mql", RestrictedHandler)
	c.POST("/mql", RoleBasedHandler)
	o.POST("/heavymql", HeavyOpenHandler)
	r.POST("/heavymql", HeavyRestrictedHandler)
	c.POST("/heavymql", HeavyRoleBasedHandler)
	jwtmdl.GlobalJWTKey = JWTKey
}

func commonHandler(c *gin.Context, isRestricted, isRoleBased, heavyDataActivity bool, principalObj servicebuildermdl.Principal) {
	serviceHeader := c.Request.Header.Get("Service-Header")
	services := strings.Split(serviceHeader, ",")
	versionError := appVersioning(c)
	if versionError != nil {
		c.JSON(http.StatusExpectationFailed, versionError.Error())
		return
	}
	responseMap := make(map[string]responseData)
	responseDataObj := responseData{}
	var reqBody []byte

	var readError error
	reqBody, readError = ioutil.ReadAll(c.Request.Body)
	if errormdl.CheckErr2(readError) != nil {
		responseDataObj.Error = errormdl.CheckErr2(readError).Error()
		responseDataObj.ErrorCode = errormdl.EXPECTATIONFAILED
		loggermdl.LogError(readError)
		c.JSON(http.StatusExpectationFailed, responseDataObj)
		return
	}

	requestBody := gjson.ParseBytes(reqBody)
	for i := 0; i < len(services); i++ {
		responseDataObj := responseData{}
		service := services[i]
		result, nextDynamicPage, ab, isCompressed, errorCode, err := executeService(service, []byte(requestBody.Get(service).String()), isRestricted, isRoleBased, heavyDataActivity, principalObj)
		if errormdl.CheckErr1(err) != nil {
			if ab == nil {
				responseDataObj.ErrorCode = errorCode
				responseDataObj.Error = err.Error()
			} else {
				responseDataObj.Error = ab.GetErrorData()
				if responseDataObj.Error == nil {
					responseDataObj.Error = err.Error()
				}
				errorCode := ab.GetErrorCode()
				if errorCode == 0 {
					errorCode = errormdl.EXPECTATIONFAILED
				}
				responseDataObj.ErrorCode = errorCode
				if ab.TransactionEnable {
					if ab.TXN != nil {
						loggermdl.LogError("Transaction Rollbacked")
						err := ab.TXN.Rollback()
						if err != nil {
							responseDataObj.Error = err.Error()
							responseDataObj.ErrorCode = errormdl.MYSQLERROR
						}
					}
				}
			}
		} else {

			if ab != nil {
				if ab.TransactionEnable {
					if ab.TXN != nil {
						loggermdl.LogError("Transaction Commit")
						err := ab.TXN.Commit()
						if err != nil {
							responseDataObj.Error = err.Error()
							responseDataObj.ErrorCode = errormdl.MYSQLERROR
						} else {
							responseDataObj.Result = result
							responseDataObj.ErrorCode = errormdl.NOERROR
						}
					} else {
						responseDataObj.Result = result
						responseDataObj.ErrorCode = errormdl.NOERROR
					}
				} else {
					responseDataObj.Result = result
					responseDataObj.ErrorCode = errormdl.NOERROR
				}
			} else {
				responseDataObj.Result = result
				responseDataObj.ErrorCode = errormdl.NOERROR
			}
		}
		responseDataObj.NextDynamicPage = nextDynamicPage
		responseDataObj.IsCompressed = isCompressed
		responseDataObj = formatResponse(ab, responseDataObj)
		responseMap[service] = responseDataObj
		// Token extraction
		if ab != nil {
			token, _ := ab.GetDataString("MQLToken")
			c.Header("Authorization", token)
		}
	}
	c.JSON(http.StatusOK, responseMap)
	return
}

// OpenHandler for /o
func OpenHandler(c *gin.Context) {
	principal := servicebuildermdl.Principal{}

	// principal.ClientIP = c.Request.RemoteAddr
	commonHandler(c, false, false, false, principal)
}

// RestrictedHandler for /r
func RestrictedHandler(c *gin.Context) {
	pricipalObj, extractError := extractPricipalObject(c)
	if extractError != nil {
		loggermdl.LogError(extractError)
		c.JSON(http.StatusExpectationFailed, extractError.Error())
		return
	}
	pricipalObj.ClientIP = c.Request.RemoteAddr
	commonHandler(c, true, false, false, pricipalObj)
}

// RoleBasedHandler for /r/c
func RoleBasedHandler(c *gin.Context) {
	pricipalObj, extractError := extractPricipalObject(c)
	if extractError != nil {
		loggermdl.LogError(extractError)
		c.JSON(http.StatusExpectationFailed, extractError.Error())
		return
	}
	pricipalObj.ClientIP = c.Request.RemoteAddr
	commonHandler(c, true, true, false, pricipalObj)
}

// HeavyOpenHandler for /o
func HeavyOpenHandler(c *gin.Context) {
	principal := servicebuildermdl.Principal{}

	// principal.ClientIP = c.Request.RemoteAddr
	commonHandler(c, false, false, true, principal)
}

// HeavyRestrictedHandler for /r
func HeavyRestrictedHandler(c *gin.Context) {
	pricipalObj, extractError := extractPricipalObject(c)
	if extractError != nil {
		loggermdl.LogError(extractError)
		c.JSON(http.StatusExpectationFailed, extractError.Error())
		return
	}
	pricipalObj.ClientIP = c.Request.RemoteAddr
	commonHandler(c, true, false, true, pricipalObj)
}

// HeavyRoleBasedHandler for /r/c
func HeavyRoleBasedHandler(c *gin.Context) {
	pricipalObj, extractError := extractPricipalObject(c)
	if extractError != nil {
		loggermdl.LogError(extractError)
		c.JSON(http.StatusExpectationFailed, extractError.Error())
		return
	}
	pricipalObj.ClientIP = c.Request.RemoteAddr
	commonHandler(c, true, true, true, pricipalObj)
}

func appVersioning(c *gin.Context) error {
	if isAppVersionEnabled {
		appVersion := c.Request.Header.Get("app-version")
		if appVersion == "" {
			return errormdl.Wrap("No App version Found in request header")
		}
		ver, err := version.NewVersion(appVersion)
		if errormdl.CheckErr(err) != nil {
			return errormdl.CheckErr(err)
		}
		if isStrictMode {
			if !ver.Equal(applicationVersion) {
				return errormdl.Wrap("Application version mismatched")
			}
		} else {
			if ver.GreaterThan(applicationVersion) {
				return errormdl.Wrap("Server Version is outdated")
			}
			if ver.LessThan(minimumSupportedVersion) {
				return errormdl.Wrap("Client Version is outdated")
			}
		}
	}
	return nil
}

func extractPricipalObject(c *gin.Context) (servicebuildermdl.Principal, error) {
	principal := servicebuildermdl.Principal{}
	if jwtmdl.GlobalJWTKey == "" {
		return principal, errormdl.Wrap("No Global JWT key found")
	}
	claim, decodeError := jwtmdl.DecodeToken(c.Request)
	if errormdl.CheckErr(decodeError) != nil {
		loggermdl.LogError(decodeError)
		return principal, errormdl.CheckErr(decodeError)
	}

	groups, grperr := roleenforcemdl.GetGroupNames(claim, "groups")
	if errormdl.CheckErr(grperr) != nil {
		loggermdl.LogError(grperr)
		return principal, errormdl.CheckErr(grperr)
	}
	userID, ok := claim["userId"].(string)
	if !ok {
		loggermdl.LogError("Unable to parse UserID from JWT Token")
		return principal, errormdl.Wrap("Unable to parse UserID from JWT Token")
	}

	if len(userID) < 2 {
		loggermdl.LogError("UserID length is less than 2")
		return principal, errormdl.Wrap("UserID length is less than 2")
	}

	rawMetadata, ok := claim["metadata"]
	if ok {
		loggermdl.LogError(rawMetadata)
		metadata, ok := rawMetadata.(string)
		if !ok {
			loggermdl.LogError("Unable to parse metadata from JWT Token")
			return principal, errormdl.Wrap("Unable to parse metadata from JWT Token")
		}
		principal.Metadata = metadata
	}
	principal.Groups = groups
	principal.UserID = userID
	principal.Token = c.Request.Header.Get("Authorization")
	return principal, nil
}
